@@ -1,5 +1,6 @@
 #define _POSIX_C_SOURCE 200809L
 
+#include <assert.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -16,6 +17,7 @@
 #define BG_COLOR 0xFFFFFF40
 #define BORDER_COLOR 0x000000FF
 #define SELECTION_COLOR 0x00000000
+#define FONT_FAMILY "sans-serif"
 
 static void noop() {
 	// This space intentionally left blank
@@ -41,6 +43,10 @@ static int32_t box_size(const struct slurp_box *box) {
 	return box->width * box->height;
 }
 
+static int max(int a, int b) {
+	return (a > b) ? a : b;
+}
+
 static int min(int a, int b) {
 	return (a < b) ? a : b;
 }
@@ -89,7 +95,7 @@ static void seat_set_outputs_dirty(struct slurp_seat *seat) {
 	wl_list_for_each(output, &seat->state->outputs, link) {
 		if (box_intersect(&output->logical_geometry,
 			&seat->pointer_selection.selection) ||
-		    box_intersect(&output->logical_geometry,
+				box_intersect(&output->logical_geometry,
 			&seat->touch_selection.selection)) {
 			set_output_dirty(output);
 		}
@@ -97,15 +103,27 @@ static void seat_set_outputs_dirty(struct slurp_seat *seat) {
 }
 
 static void handle_active_selection_motion(struct slurp_seat *seat, struct slurp_selection *current_selection) {
+	if(seat->state->restrict_selection){
+		return;
+	}
+
 	int32_t anchor_x = current_selection->anchor_x;
 	int32_t anchor_y = current_selection->anchor_y;
-	current_selection->selection.x = min(anchor_x, current_selection->x);
-	current_selection->selection.y = min(anchor_y, current_selection->y);
+	int32_t dist_x = current_selection->x - anchor_x;
+	int32_t dist_y = current_selection->y - anchor_y;
+
+	current_selection->has_selection = true;
 	// selection includes the seat and anchor positions
-	current_selection->selection.width =
-		abs(current_selection->x - anchor_x) + 1;
-	current_selection->selection.height =
-		abs(current_selection->y - anchor_y) + 1;
+	int32_t width = abs(dist_x) + 1;
+	int32_t height = abs(dist_y) + 1;
+	if (seat->state->aspect_ratio) {
+		width = max(width, height / seat->state->aspect_ratio);
+		height = max(height, width * seat->state->aspect_ratio);
+	}
+	current_selection->selection.x = dist_x > 0 ? anchor_x : anchor_x - (width - 1);
+	current_selection->selection.y = dist_y > 0 ? anchor_y : anchor_y - (height - 1);
+	current_selection->selection.width = width;
+	current_selection->selection.height = height;
 }
 
 static void pointer_handle_enter(void *data, struct wl_pointer *wl_pointer,
@@ -116,6 +134,12 @@ static void pointer_handle_enter(void *data, struct wl_pointer *wl_pointer,
 	if (output == NULL) {
 		return;
 	}
+
+	// the places the cursor moved away from are also dirty
+	if (seat->pointer_selection.has_selection) {
+		seat_set_outputs_dirty(seat);
+	}
+
 	// TODO: handle multiple overlapping outputs
 	seat->pointer_selection.current_output = output;
 
@@ -188,7 +212,6 @@ static void handle_selection_start(struct slurp_seat *seat,
 			state->running = false;
 		}
 	} else {
-		current_selection->has_selection = true;
 		current_selection->anchor_x = current_selection->x;
 		current_selection->anchor_y = current_selection->y;
 	}
@@ -259,6 +282,15 @@ static void keyboard_handle_keymap(void *data, struct wl_keyboard *wl_keyboard,
 	seat->xkb_state = xkb_state_new(seat->xkb_keymap);
 }
 
+// Recompute the selection if the aspect ratio changed.
+static void recompute_selection(struct slurp_seat *seat) {
+	struct slurp_selection *current = slurp_seat_current_selection(seat);
+	if (current->has_selection) {
+		handle_active_selection_motion(seat, slurp_seat_current_selection(seat));
+		seat_set_outputs_dirty(seat);
+	}
+}
+
 static void keyboard_handle_key(void *data, struct wl_keyboard *wl_keyboard,
 		const uint32_t serial, const uint32_t time, const uint32_t key,
 		const uint32_t key_state) {
@@ -283,12 +315,22 @@ static void keyboard_handle_key(void *data, struct wl_keyboard *wl_keyboard,
 			}
 			state->edit_anchor = true;
 			break;
+		case XKB_KEY_Shift_L:
+		case XKB_KEY_Shift_R:
+			if (!state->fixed_aspect_ratio) {
+				state->aspect_ratio = 1;
+				recompute_selection(seat);
+			}
+			break;
 		}
 		break;
 
 	case WL_KEYBOARD_KEY_STATE_RELEASED:
 		if (keysym == XKB_KEY_space) {
 			state->edit_anchor = false;
+		} else if (!state->fixed_aspect_ratio && (keysym == XKB_KEY_Shift_L || keysym == XKB_KEY_Shift_R)) {
+			state->aspect_ratio = 0;
+			recompute_selection(seat);
 		}
 		break;
 	}
@@ -531,6 +573,10 @@ static void send_frame(struct slurp_output *output) {
 	if (output->current_buffer == NULL) {
 		return;
 	}
+	output->current_buffer->busy = true;
+
+	cairo_identity_matrix(output->current_buffer->cairo);
+	cairo_scale(output->current_buffer->cairo, output->scale, output->scale);
 
 	render(output);
 
@@ -652,11 +698,13 @@ static const char usage[] =
 	"  -c #rrggbbaa Set border color.\n"
 	"  -s #rrggbbaa Set selection color.\n"
 	"  -B #rrggbbaa Set option box color.\n"
+	"  -F s         Set the font family for the dimensions.\n"
 	"  -w n         Set border weight.\n"
 	"  -f s         Set output format.\n"
 	"  -o           Select a display output.\n"
 	"  -p           Select a single point.\n"
-	"  -r           Restrict selection to predefined boxes.\n";
+	"  -r           Restrict selection to predefined boxes.\n"
+	"  -a w:h       Force aspect ratio.\n";
 
 uint32_t parse_color(const char *color) {
 	if (color[0] == '#') {
@@ -676,24 +724,32 @@ uint32_t parse_color(const char *color) {
 	return res;
 }
 
-static void print_output_name(FILE *stream, const struct slurp_box *result, struct wl_list *outputs) {
+static struct slurp_output *output_from_box(const struct slurp_box *box, struct wl_list *outputs) {
 	struct slurp_output *output;
 	wl_list_for_each(output, outputs, link) {
-		// For now just use the top-left corner
 		struct slurp_box *geometry = &output->logical_geometry;
-		if (in_box(geometry, result->x, result->y)) {
-			if (geometry->label) {
-				fprintf(stream, "%s", geometry->label);
-				return;
-			}
-			break;
+		// For now just use the top-left corner
+		if (in_box(geometry, box->x, box->y)) {
+			return output;
+		}
+	}
+	return NULL;
+}
+
+static void print_output_name(FILE *stream, const struct slurp_box *result, struct wl_list *outputs) {
+	struct slurp_output *output = output_from_box(result, outputs);
+	if (output) {
+		struct slurp_box *geometry = &output->logical_geometry;
+		if (geometry->label) {
+			fprintf(stream, "%s", geometry->label);
+			return;
 		}
 	}
 	fprintf(stream, "<unknown>");
 }
 
-static void print_formatted_result(FILE *stream, const struct slurp_box *result, struct wl_list *outputs,
-		const char *format) {
+static void print_formatted_result(FILE *stream, struct slurp_state *state , const char *format) {
+	struct slurp_output *output = output_from_box(&state->result, &state->outputs);
 	for (size_t i = 0; format[i] != '\0'; i++) {
 		char c = format[i];
 		if (c == '%') {
@@ -702,24 +758,40 @@ static void print_formatted_result(FILE *stream, const struct slurp_box *result,
 			i++; // Skip the next character (x, y, w or h)
 			switch (next) {
 			case 'x':
-				fprintf(stream, "%d", result->x);
+				fprintf(stream, "%d", state->result.x);
 				continue;
 			case 'y':
-				fprintf(stream, "%d", result->y);
+				fprintf(stream, "%d", state->result.y);
 				continue;
 			case 'w':
-				fprintf(stream, "%d", result->width);
+				fprintf(stream, "%d", state->result.width);
 				continue;
 			case 'h':
-				fprintf(stream, "%d", result->height);
+				fprintf(stream, "%d", state->result.height);
+				continue;
+			case 'X':
+				assert(output);
+				fprintf(stream, "%d", state->result.x - output->logical_geometry.x);
+				continue;
+			case 'Y':
+				assert(output);
+				fprintf(stream, "%d", state->result.y - output->logical_geometry.y);
+				continue;
+			case 'W':
+				assert(output);
+				fprintf(stream, "%d", min(state->result.width, output->logical_geometry.x + output->logical_geometry.width - state->result.x));
+				continue;
+			case 'H':
+				assert(output);
+				fprintf(stream, "%d", min(state->result.height, output->logical_geometry.y + output->logical_geometry.height - state->result.y));
 				continue;
 			case 'l':
-				if (result->label) {
-					fprintf(stream, "%s", result->label);
+				if (state->result.label) {
+					fprintf(stream, "%s", state->result.label);
 				}
 				continue;
 			case 'o':
-				print_output_name(stream, result, outputs);
+				print_output_name(stream, &state->result, &state->outputs);
 				continue;
 			default:
 				// If no case was executed, revert i back - we don't need to
@@ -763,12 +835,16 @@ int main(int argc, char *argv[]) {
 		.border_weight = 2,
 		.display_dimensions = false,
 		.restrict_selection = false,
+		.fixed_aspect_ratio = false,
+		.aspect_ratio = 0,
+		.font_family = FONT_FAMILY
 	};
 
 	int opt;
 	char *format = "%x,%y %wx%h\n";
 	bool output_boxes = false;
-	while ((opt = getopt(argc, argv, "hdb:c:s:B:w:prof:")) != -1) {
+	int w, h;
+	while ((opt = getopt(argc, argv, "hdb:c:s:B:w:proa:f:F:")) != -1) {
 		switch (opt) {
 		case 'h':
 			printf("%s", usage);
@@ -791,6 +867,9 @@ int main(int argc, char *argv[]) {
 		case 'f':
 			format = optarg;
 			break;
+		case 'F':
+			state.font_family = optarg;
+			break;
 		case 'w': {
 			errno = 0;
 			char *endptr;
@@ -810,6 +889,18 @@ int main(int argc, char *argv[]) {
 		case 'r':
 			state.restrict_selection = true;
 			break;
+		case 'a':
+			if (sscanf(optarg, "%d:%d", &w, &h) != 2) {
+				fprintf(stderr, "invalid aspect ratio\n");
+				return EXIT_FAILURE;
+			}
+			if (w <= 0 || h <= 0) {
+				fprintf(stderr, "width and height of aspect ratio must be greater than zero\n");
+				return EXIT_FAILURE;
+			}
+			state.fixed_aspect_ratio = true;
+			state.aspect_ratio = (double) h / w;
+			break;
 		default:
 			printf("%s", usage);
 			return EXIT_FAILURE;
@@ -965,7 +1056,7 @@ int main(int argc, char *argv[]) {
 		fprintf(stderr, "selection cancelled\n");
 		status = EXIT_FAILURE;
 	} else {
-		print_formatted_result(stream, &state.result, &state.outputs, format);
+		print_formatted_result(stream, &state, format);
 		fclose(stream);
 	}
 
