@@ -13,10 +13,10 @@ static void set_source_u32(cairo_t *cairo, uint32_t color) {
 		(color >> (0 * 8) & 0xFF) / 255.0);
 }
 
-static void draw_rect(cairo_t *cairo, struct slurp_box *box, uint32_t color, int32_t scale) {
+static void draw_rect(cairo_t *cairo, struct slurp_box *box, uint32_t color) {
 	set_source_u32(cairo, color);
-	cairo_rectangle(cairo, box->x * scale, box->y * scale,
-			box->width * scale, box->height * scale);
+	cairo_rectangle(cairo, box->x, box->y,
+			box->width, box->height);
 }
 
 static void box_layout_to_output(struct slurp_box *box, struct slurp_output *output) {
@@ -28,7 +28,6 @@ void render(struct slurp_output *output) {
 	struct slurp_state *state = output->state;
 	struct pool_buffer *buffer = output->current_buffer;
 	cairo_t *cairo = buffer->cairo;
-	int32_t scale = output->scale;
 
 	// Clear
 	cairo_set_operator(cairo, CAIRO_OPERATOR_SOURCE);
@@ -42,7 +41,7 @@ void render(struct slurp_output *output) {
 					choice_box)) {
 			struct slurp_box b = *choice_box;
 			box_layout_to_output(&b, output);
-			draw_rect(cairo, &b, state->colors.choice, scale);
+			draw_rect(cairo, &b, state->colors.choice);
 			cairo_fill(cairo);
 		}
 	}
@@ -50,9 +49,7 @@ void render(struct slurp_output *output) {
 	struct slurp_seat *seat;
 	wl_list_for_each(seat, &state->seats, link) {
 		struct slurp_selection *current_selection =
-			seat->touch_selection.has_selection ?
-				&seat->touch_selection :
-				&seat->pointer_selection;
+			slurp_seat_current_selection(seat);
 
 		if (!current_selection->has_selection) {
 			continue;
@@ -65,25 +62,26 @@ void render(struct slurp_output *output) {
 		struct slurp_box b = current_selection->selection;
 		box_layout_to_output(&b, output);
 
-		draw_rect(cairo, &b, state->colors.selection, scale);
+		draw_rect(cairo, &b, state->colors.selection);
 		cairo_fill(cairo);
 
 		// Draw border
-		cairo_set_line_width(cairo, state->border_weight * scale);
-		draw_rect(cairo, &b, state->colors.border, scale);
+		cairo_set_line_width(cairo, state->border_weight);
+		draw_rect(cairo, &b, state->colors.border);
 		cairo_stroke(cairo);
 
 		if (state->display_dimensions) {
-			cairo_select_font_face(cairo, "Sans",
+			cairo_select_font_face(cairo, state->font_family,
 					       CAIRO_FONT_SLANT_NORMAL,
 					       CAIRO_FONT_WEIGHT_NORMAL);
-			cairo_set_font_size(cairo, 14 * scale);
+			cairo_set_font_size(cairo, 14);
+			set_source_u32(cairo, state->colors.border);
 			// buffer of 12 can hold selections up to 99999x99999
 			char dimensions[12];
 			snprintf(dimensions, sizeof(dimensions), "%ix%i",
 				 b.width, b.height);
-			cairo_move_to(cairo, (b.x + b.width + 10) * scale,
-				      (b.y + b.height + 20) * scale);
+			cairo_move_to(cairo, b.x + b.width + 10,
+				      b.y + b.height + 20);
 			cairo_show_text(cairo, dimensions);
 		}
 	}
